#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>
#include "IOTCDevice.h"
#include "sample_constants.h"

typedef struct _targetChannel
{
	int nSID;
	int ChID;
}TargetChannel;

#define MAX_SESSION 8

void *thread_ForSessionReadWrite(void *arg)
{
	TargetChannel *pTargetChl = (TargetChannel *)arg;
	unsigned char ChID = pTargetChl->ChID;
	int SID = pTargetChl->nSID;
	int i;
	int nRead;
	char buf[IOTC_MAX_PACKET_SIZE];
	struct st_SInfoEx Sinfo;

	printf("[thread_ForSessionReadWrite] Thread started...[%d]\n", ChID);

    Sinfo.size = sizeof(struct st_SInfoEx);
	IOTC_Session_Check_Ex(SID, &Sinfo);
	if(Sinfo.Mode ==0)
		printf("Client is from %s:%d[%s] Mode=P2P\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);
	else if(Sinfo.Mode == 1)
		printf("Client is from %s:%d[%s] Mode=RLY\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);
	else if(Sinfo.Mode == 2)
		printf("Client is from %s:%d[%s] Mode=LAN\n",Sinfo.RemoteIP, Sinfo.RemotePort, Sinfo.UID);

	printf("VPG [%d:%d:%d] Remote NAT[%d]\n", Sinfo.VID, Sinfo.PID, Sinfo.GID, Sinfo.RemoteNatType);

	IOTC_Session_Channel_ON(SID, ChID);
	sleep(3);

	for(i = 0; i < 3; i++)
	{
		sprintf(buf, "Hello World[%d][%d]", ChID, i);
		int ret = IOTC_Session_Write(SID,buf,strlen(buf), ChID);
		if(ret < 0)
			printf("Write error[%d]->%d\n", ChID, ret);
	}

	sprintf(buf, "end");
	IOTC_Session_Write(SID, buf, strlen(buf), ChID);

	while((nRead = IOTC_Session_Read(SID, buf, IOTC_MAX_PACKET_SIZE, 6000, ChID)) > -1)
	{
		buf[nRead] = 0;
		printf("[IOTC_Session_Read] SID=%d, ChID = %d, Size=%d, Data:%s\n",SID,ChID,nRead,buf);
	}
	// check if remote site close this session
	if(nRead == IOTC_ER_SESSION_CLOSE_BY_REMOTE)
	{
		printf("[thread_ForSessionReadWrite] %d remote site close this session, SID = %d\n", ChID, SID);
		IOTC_Session_Close(SID);
	}
	else if(nRead == IOTC_ER_REMOTE_TIMEOUT_DISCONNECT)
	{
		printf("[thread_ForSessionReadWrite] %d disconnected due to remote site has no any response after a 'timeout' period of time., SID = %d\n", ChID, SID);
		IOTC_Session_Close(SID);
	}
	else
	{
		printf("[thread_ForSessionReadWrite] %d, nRead = %d, SID = %d\n", ChID, nRead, SID);
		IOTC_Session_Close(SID);
	}

	printf("[thread_ForSessionRead] Thread exit...[%d]\n", ChID);
	free(pTargetChl);
	pthread_exit(0);
}

DeviceLoginInput auth_option;
int use_key = 0;

void *_thread_Login(void *arg)
{
    int ret;
    char *UID = (char *)arg;

    while (1)
	{
		if (use_key)
		{
			ret = IOTC_Device_LoginEx(UID, &auth_option);
			printf("     Calling IOTC_Device_LoginEX() ret = %d\n", ret);
		}
		else
		{
			ret = IOTC_Device_Login(UID, NULL,NULL);
			printf("     Calling IOTC_Device_Login() ret = %d\n", ret);
		}

        if (ret == IOTC_ER_NoERROR)
            break;
        else
            sleep(5);
    }

    pthread_exit(0);
}

int main(int argc, char *argv[])
{
	int ret;
	const unsigned char ChID0 = 0, ChID1 = 1;
	char *UID = (char *)argv[1];
	pthread_t threadID_Login;

	if(argc < 2)
	{
		printf("No UID input!!!\n");
		printf("IOTCAPIs_Device [UID]\n");
		return 0;
	}

    auth_option.cb = sizeof(DeviceLoginInput);
    auth_option.authentication_type = AUTHENTICATE_BY_KEY;
	memset(auth_option.auth_key, '0', 8);

    if(argc > 2)
	{
		if(strlen(argv[2]) == 8)
		{
			strncpy(auth_option.auth_key, argv[2], 8);
			printf("Login with Key=%s\n", argv[2]);
			use_key = 1;
		}
		else
		{
			printf("Invalid Key=%s, Login without Key\n", argv[2]);
		}
	}
	else
	{
		printf("Login without Key\n");
	}

    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));

    log_attr.path = "./log_iotc_device.txt";
    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_global_device.txt";
    ret = TUTK_Set_Log_Attr(log_attr);
    printf("TUTK_Set_Log_Attr ret[%d]\n", ret);

    const char *license_key = getenv(ENV_LICENSE_KEY);
    if (license_key == NULL) {
        printf("Environment variable %s must be set!!\n", ENV_LICENSE_KEY);
        return -1;
    }
    ret = TUTK_SDK_Set_License_Key(license_key);
    if (ret != TUTK_ER_NoERROR)
    {
        printf("TUTK_SDK_Set_License_Key()=%d ,exit...!!\n", ret);
        return 0;
    }

    // Must be first call before IOTC_Initialize2(), but not a must
	IOTC_Set_Max_Session_Number(MAX_SESSION);

	printf("IOTCAPIs_Device start...\n");
	ret = IOTC_Initialize2(0);
	printf("Step 1: IOTC_Initialize2() ret = %d\n", ret);
	if(ret != IOTC_ER_NoERROR)
	{
		printf("IOTCAPIs_Device exit...!!\n");
		return 0;
	}

	pthread_create(&threadID_Login, NULL, &_thread_Login, (void *)UID);
	pthread_detach(threadID_Login);

	while(1)
	{
		pthread_t Thread_ID1, Thread_ID2;
		int nSID = IOTC_Listen(0);

		if(nSID > -1)
		{
			TargetChannel *pTargetChl0 = (TargetChannel *)malloc(sizeof(TargetChannel));
			pTargetChl0->nSID = nSID;
			pTargetChl0->ChID = ChID0;

			TargetChannel *pTargetChl1 = (TargetChannel *)malloc(sizeof(TargetChannel));
			pTargetChl1->nSID = nSID;
			pTargetChl1->ChID = ChID1;

			printf("Step 3: IOTC_Listen() ret = %d ", nSID);

			printf("Step4: Create a Thread to handle data read / write from this session\n");

			pthread_create(&Thread_ID1, NULL, &thread_ForSessionReadWrite, (void*)pTargetChl0);
			pthread_create(&Thread_ID2, NULL, &thread_ForSessionReadWrite, (void*)pTargetChl1);
			pthread_detach(Thread_ID1);
			pthread_detach(Thread_ID2);
		}
		else if(nSID == IOTC_ER_EXCEED_MAX_SESSION)
		{
			printf("IOTC_ER_EXCEED_MAX_SESSION\n");
			sleep(5);
		}
		else
		{
			printf("IOTC_Listen--Error[%d]\n", nSID);
			printf("IOTCAPIs_Device exit...!!\n");
			break;
		}
	}

	IOTC_DeInitialize();

	return 0;
}

